package com.dzeio.openhealth.ui.water

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.charts.Entry
import com.dzeio.charts.series.BarSerie
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.ItemAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.databinding.FragmentMainWaterHomeBinding
import com.dzeio.openhealth.units.Units
import com.dzeio.openhealth.utils.ChartUtils
import dagger.hilt.android.AndroidEntryPoint
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

@AndroidEntryPoint
class WaterHomeFragment :
    BaseFragment<WaterHomeViewModel, FragmentMainWaterHomeBinding>(WaterHomeViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentMainWaterHomeBinding =
        FragmentMainWaterHomeBinding::inflate

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewModel.init()

        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = ItemAdapter<Water>().apply {
            onItemClick = {
                findNavController().navigate(
                    WaterHomeFragmentDirections.actionNavWaterHomeToNavWaterEdit(
                        it.value.id
                    )
                )
            }
            recycler.adapter = this
        }

        val chart = binding.chart

        val serie = BarSerie(chart)

        chart.apply {
            ChartUtils.materielTheme(chart, requireView())

            yAxis.apply {
//                onValueFormat
            }

            xAxis.apply {
                dataWidth = 604800000.0
                textPaint.textSize = 32f
                scrollEnabled = true
                onValueFormat = onValueFormat@{
                    return@onValueFormat SimpleDateFormat(
                        "yyyy-MM-dd",
                        Locale.getDefault()
                    ).format(Date(it.toLong()))
                }
            }
        }

        binding.buttonEditDefaultIntake.setOnClickListener {
            findNavController().navigate(
                WaterHomeFragmentDirections.actionNavWaterHomeToNavWaterSizeDialog()
            )
        }

        viewModel.items.observe(viewLifecycleOwner) { list ->
            val unit = Units.Volume.MILLILITER
            adapter.set(
                list.map {
                    ItemAdapter.Item(
                        it,
                        getString(
                            unit.unit,
                            unit.formatToString(it.value)
                        ),
                        it.formatTimestamp(),
                        icon = R.drawable.ic_outline_edit_24
                    )
                }
            )

            if (list.isEmpty()) {
                return@observe
            }

            val dataset = list.map {
                return@map Entry(
                    it.timestamp.toDouble(),
                    it.value.toFloat()
                )
            }

            serie.entries = dataset as ArrayList<Entry>
            chart.xAxis.x = chart.xAxis.getXMin()
            chart.xAxis.x =
                chart.xAxis.getXMax() - chart.xAxis.dataWidth!! + chart.xAxis.dataWidth!! / 7
            chart.refresh()
        }
    }
}
